      MODULE ncplot_input_module
! module to set postprocessed reference times and comment lines
! for IGOR-friendly GECKO box model output
!======================================================================
! USER SUPPLIES OUTPUT TIME DEFINITIONS : REQUIRED PARAMS ARE....
!      INTEGER,PARAMETER :: nrt = (value or equation)
!      REAL,DIMENSION(nrt) :: reftime = (values or expression)
!      CHARACTER(LEN=50) :: creftime = (string)
!======================================================================
      IMPLICIT NONE
      INTEGER :: i,ii,iii,iv

! HEADER COMMENT LINE (up to 70 chars, info about the run)
      !CHARACTER(LEN=30):: notes1 = 'dtmin=0.1 (default) '
      CHARACTER(LEN=30):: notes1 = 'dtmin 0.001 '
      !CHARACTER(LEN=30):: notes1 = 'dtmin=0.01 (OFR) '
      !CHARACTER(LEN=30):: notes1 = 'dtmin=0.1, tlen=300 '

! model output times: stop, and timestep, both in s
      INTEGER,PARAMETER :: opstrt = 0.  ! units = seconds
      !INTEGER,PARAMETER :: opstop = 10.  ! units = seconds
      INTEGER,PARAMETER :: opstop = 180.  ! units = seconds
      REAL,PARAMETER    :: opstep = 0.01   ! units = seconds
      !INTEGER,PARAMETER :: opstop = 864000.  ! units = seconds
      !REAL,PARAMETER    :: opstep = 3600.   ! units = seconds

!--------------------------------------------------------------------
!--EITHER calculate postprocessed output time array (evenly spaced)
!--------------------------------------------------------------------
!      INTEGER,PARAMETER :: rtstrt = 0.
!      INTEGER,PARAMETER :: rtstop = 10.
!      REAL,PARAMETER    :: rtstep = 0.1   ! units = seconds
!      !CHARACTER(LEN=50) :: creftime="0-24h x 10m"
!      CHARACTER(LEN=50) :: creftime="0-10s x 0.1s"
!      INTEGER,PARAMETER :: nrt=INT(FLOAT(rtstop-rtstrt)/rtstep)+1
!      REAL,DIMENSION(nrt) :: reftime=rtstrt+(rtstep*(/(i,i=0,nrt-1,1)/))

!--------------------------------------------------------------------
!--OR specify variable spacing output time array (nrt must agree)
!--------------------------------------------------------------------
!  ZP 180s OFR run: 0-1s @0.01s; 10-30, 30-120 mins @1,2 mins
!              Model OP TS = 0.01s
      CHARACTER(LEN=50) :: creftime=  &
      "0-1s,1-10s,10-30s,30-180s x 0.01s,0.1s,1s,10s"
      INTEGER,PARAMETER :: nrt = 226
      REAL,DIMENSION(nrt) :: reftime = (/ &
         (  i,  i=   0,   99,   1),  &! 100
         ( ii, ii= 100,  999,  10),  &!  89
         (iii,iii=1000, 2999, 100),  &!  29
         ( iv, iv=3000,18000,1000)   &!  15
                                /)
! WB 2hr CHAM run: 0-10min @10s; 10-30, 30-120 mins @1,2 mins
!!              Model OP TS = 10s
!      CHARACTER(LEN=50) :: creftime=  &
!         "0-10m, 10-30m, 30-120m x 10s, 1 min, 2i min"
!      INTEGER,PARAMETER :: nrt = 126
!      REAL,DIMENSION(nrt) :: reftime = (/ &
!         (  i,  i=    0,   590,  10),  &! 60
!         ( ii, ii=  600,  1740,  60),  &! 20
!         (iii,iii= 1800,  7200, 120)   &! 46
!                             /)
! ZP 30 min OHR run: 0-1s @0.01s; 1-2s @0.1s; 2s-3mins @1s; 3-10min @2s;
!!             10-30min @10s. Model OP TS = 0.01s
!      CHARACTER(LEN=50) :: creftime=
!     &"0-1s,1-2s,2s-1m,1-10m,10-30m x 0.01,0.1,1,2s,10s"
!      INTEGER,PARAMETER :: nrt = 469
!      REAL,DIMENSION(nrt) :: reftime = rtstep * (/
!     &   (i,i=    0,    99,  10),
!     &   (j,j=  100,   199,  10),
!     &   (k,k=  200,  5999, 100),
!     &   (l,l= 6000, 59999, 200),
!     &   (m,m=60000,180000,1000)
!     &                        /)
!--------------------------------------------------------------------

      END MODULE ncplot_input_module
