!***********************************************************************
* MASTER MECHANISM - ROUTINE NAME :      SMILES                        *
*                                                                      *
* PURPOSE:  -  convert standard chemical formula to SMILES format      *
*                                                                      *
* INPUT                                                                *
* - chem         : standard formula of species                         *
*                                                                      *
* OUTPUT                                                               *
* - smile        : SMILES-notation formula in same orientation as chem *
*
************************************************************************
      SUBROUTINE smiles(chem,smile)
      IMPLICIT NONE
      INCLUDE 'smilesvars.h'

* input:
      CHARACTER*(lfo) chem
* ouput:
      CHARACTER*(lfo) smile

* local
      INTEGER         cnum
      INTEGER         onum
      INTEGER         dbflg
      INTEGER         bond(mca,mca)
      INTEGER         path(mca,mca,mca)
      INTEGER         clngth(mca,mca)
      INTEGER         rjg(mri,2)
      CHARACTER*(lgr) group(mca),tgrp(mca)
      CHARACTER*(lgr) grpnew,grpold
      LOGICAL         lobond(mca,mca)

      CHARACTER*(lfo) tsmile,tchem,copy(mco)
      INTEGER         nc, nca, nce, nring 
      INTEGER         leaf, last, ptr, p, i, j, k, ncp
      INTEGER         ig, pg, ng
      INTEGER         kn,ko,kg
      INTEGER         rank(mca)
      INTEGER         ncbn,nhyd,noxy,nnit

* ------------------------------------------------------
* INITIALIZE
* ------------------------------------------------------
!      print*,'subroutine smiles'

      ncbn = 0
      nhyd = 0
      noxy = 0
      nnit = 0
      smile(1:lfo)=" "

      DO i=1,mco
        copy(i) = ' '
      ENDDO

      DO i=1,mca
        DO j=1,mca
          clngth(i,j) = 0
          lobond(i,j) = .false.
          DO k=1,mca
            path(i,j,k) = 0
          ENDDO
        ENDDO
      ENDDO


* ------------------------------------------------------
* CHECK INPUT FORMULA
* ------------------------------------------------------
* CHECK FOR RADICALS (not represented)
      IF(INDEX(chem,'.').NE.0)THEN
        smile = 'N/A'
        RETURN
      ENDIF

* CHECK FOR '#mm' notation
      IF(INDEX(chem,'#mm').NE.0)THEN

* USER ALERT : code currently cannot handle aromatics
        smile = 'ERROR:bug_in_SMILES_for_aromatics'
        RETURN
* END USER ALERT

        tchem=chem(4:lfo)//'   '
        chem=tchem
!        print*,chem
      ELSE
        IF(INDEX(chem,'#').NE.0)THEN
          PRINT*,"species ",chem," cannot be parsed by smiles"
          RETURN
        ENDIF
      ENDIF

* get the number of >C< , >c< (nca) and -O- (nce) in the molecule
      nc  = INDEX(chem,' ') - 1
      ncbn = cnum(chem,nc)
      nce = onum(chem,nc)
      nca = ncbn + nce

* USER_ALERT: code currently cannot handle C16+
      IF (nca.GE.16) THEN
        smile = 'ERROR:bug_in_SMILES_for_C#>16'
        RETURN
      ENDIF
* END USE ALERT

* only multi-node molecules checked:
      IF (nca.LE.1) THEN
        smile = 'N/A'
        RETURN
      ENDIF

* ------------------------------------------------------
* build the bond and group matrix
* ------------------------------------------------------
      CALL grbond(chem,nc,group,bond,dbflg,nring)

* USER ALERT: code currently cannot handle C16+
      IF (nring.GE.1) THEN
        smile = 'ERROR:bug_in_SMILES_for_cyclics'
        RETURN
      ENDIF
* END USER ALERT

      CALL ratings(nca,group,bond,nring,rank)
      IF(nring.GT.0) THEN
        CALL uniqring(nring,nca,group,bond,rank,rjg)
        DO k=1,nring
          i=rjg(k,1)
          j=rjg(k,2)
          bond(i,j) = 0
          bond(j,i) = 0
        ENDDO
      ENDIF


      !DO i=1,nca
      !  PRINT*,group(i)      
      !ENDDO
      !PRINT*,'----------'

      IF(nring.GT.0) CALL rjgrm(nring,group,rjg)
 
* ------------------------------------------------------
* convert individual groups, counting atoms as you go.
* ------------------------------------------------------
      DO i=1,nca
        !PRINT*,group(i)      
        tgrp(i)=group(i)

* a) remove CH* hydrogens
        IF(INDEX(tgrp(i),'CH').NE.0.OR.INDEX(tgrp(i),'cH').NE.0)THEN
          ptr=MAX(INDEX(tgrp(i),'CH')+1,INDEX(tgrp(i),'cH')+1)
          IF(INDEX(tgrp(i),'CH2').NE.0)THEN
            k=2
            nhyd = nhyd+2
          ELSE IF(INDEX(tgrp(i),'CH3').NE.0)THEN
            k=2
            nhyd = nhyd+3
          ELSE
            k=1
            nhyd = nhyd+1
          ENDIF
          tgrp(i)(ptr:mca-k)=tgrp(i)(ptr+k:mca)
          !PRINT*,"a)",tgrp(i)(1:INDEX(tgrp(i)," ")),ncbn,nhyd,noxy,nnit
        ENDIF

* b) remove CdH hydrogens
        IF(INDEX(tgrp(i),'CdH').NE.0)THEN
          ptr=INDEX(tgrp(i),'CdH')+2
          IF(INDEX(tgrp(i),'CdH2').NE.0)THEN
            k=2
            nhyd = nhyd+2
          ELSE
            k=1
            nhyd = nhyd+1
          ENDIF
          tgrp(i)(ptr:mca-k)=tgrp(i)(ptr+k:mca)
          !PRINT*,"b)",tgrp(i)(1:INDEX(tgrp(i)," ")),ncbn,nhyd,noxy,nnit
        ENDIF

* c) convert NO2 to N(=O)=O {N,P,V}
        IF(INDEX(tgrp(i),'ONO2').NE.0)THEN
          noxy = noxy + 1
        ENDIF
        IF(INDEX(tgrp(i),'NO2').NE.0)THEN
          noxy = noxy + 2
          nnit = nnit + 1
          grpold = 'NO2'
          grpnew = 'N(=O)=O'
          ko=LEN(TRIM(grpold))
          kn=LEN(TRIM(grpnew))

          ptr = INDEX(tgrp(i),TRIM(grpold))
          kg = LEN(tgrp(i))
          tgrp(i)(ptr+kn:kg+kn-ko)=tgrp(i)(ptr+ko:kg)
          tgrp(i)(ptr:ptr+kn-1)=grpnew
          !PRINT*,"c)",tgrp(i)(1:INDEX(tgrp(i)," ")),ncbn,nhyd,noxy,nnit
        ENDIF

* d) convert CO to C(=O) {A,G,P} 
        IF(INDEX(tgrp(i),'CO').NE.0)THEN
          noxy = noxy + 1
          grpold = 'CO'
          grpnew = 'C(=O)'
          ko=LEN(TRIM(grpold))
          kn=LEN(TRIM(grpnew))

          ptr = INDEX(tgrp(i),TRIM(grpold))
          kg = LEN(tgrp(i))
          tgrp(i)(ptr+kn:kg+kn-ko)=tgrp(i)(ptr+ko:kg)
          tgrp(i)(ptr:ptr+kn-1)=grpnew
          !PRINT*,"d)",tgrp(i)(1:INDEX(tgrp(i)," ")),ncbn,nhyd,noxy,nnit
        ENDIF

* e) convert OH to O {A,G,H,O} 
        IF(INDEX(tgrp(i),'(OOH)').NE.0)THEN
          noxy = noxy + 2
          nhyd = nhyd + 1
        ENDIF
        IF(INDEX(tgrp(i),'(OH)').NE.0)THEN
          noxy = noxy + 1
          nhyd = nhyd + 1
        ENDIF
        IF(INDEX(tgrp(i),'OH').NE.0)THEN
          grpold = 'OH'
          grpnew = 'O'
          ko=LEN(TRIM(grpold))
          kn=LEN(TRIM(grpnew))

          ptr = INDEX(tgrp(i),TRIM(grpold))
          kg = LEN(tgrp(i))
          tgrp(i)(ptr+kn:kg+kn-ko)=tgrp(i)(ptr+ko:kg)
          tgrp(i)(ptr:ptr+kn-1)=grpnew
          !PRINT*,"e)",tgrp(i)(1:INDEX(tgrp(i)," ")),ncbn,nhyd,noxy,nnit
        ENDIF
        
* f) convert -O- to O {E} 
        IF(INDEX(tgrp(i),'-O-').NE.0)THEN
          noxy = noxy + 1
          grpold = '-O-'
          grpnew = 'O'
          ko=LEN(TRIM(grpold))
          kn=LEN(TRIM(grpnew))

          ptr = INDEX(tgrp(i),TRIM(grpold))
          kg = LEN(tgrp(i))
          tgrp(i)(ptr+kn:kg+kn-ko)=tgrp(i)(ptr+ko:kg)
          tgrp(i)(ptr:ptr+kn-1)=grpnew
          !PRINT*,"f)",tgrp(i)(1:INDEX(tgrp(i)," ")),ncbn,nhyd,noxy,nnit
        ENDIF
        

      ENDDO
* ------------------------------------------------------
* add ring chars back in
      IF(nring.GT.0) CALL rjgadd(nring,tgrp,rjg)

* ------------------------------------------------------
* make a logical copy of the bond matrix
      DO i=1,nca
        DO j=1,nca
          IF (bond(i,j).NE.0) lobond(i,j)= .true.
        ENDDO
      ENDDO

* find longest tree, top-down, starting with the first group
      CALL lntree(bond,1,2,nca,clngth,path)

* look down-top for the very longest tree ...
      ncp = 0
      DO 50 i=1,nca
        IF (clngth(1,i).NE.0) THEN
           leaf = path(1,i,clngth(1,i))
           last = path(1,i,clngth(1,i)-1)
           CALL lntree(bond,leaf,last,nca,clngth,path)

* write the SMILES of species according to the longest tree 
* in path. Each group are written in the formula with all the
* branches by the mkcopy subroutine. OMIT revers subroutine. (Should be
* in standard format already.) 
           DO 55 j=1,nca
             IF (clngth(leaf,j).NE.0) THEN
                ncp=ncp+1
                ptr = 1
                DO k=1,nca
                  ig = path(leaf,j,k)
                  IF (ig.NE.0) THEN
                     IF (k.GT.1)   pg = path(leaf,j,k-1)
                     IF (k.LT.nca) ng = path(leaf,j,k+1)
        CALL mkcopy (lobond,tgrp,nca,rank,nring,ig,pg,ng,ptr,copy(ncp))
                  ENDIF
                ENDDO
             ENDIF
55         CONTINUE
        ENDIF
50    CONTINUE

* just use the first available version - SMILES doesn't care
      tsmile = copy(ncp)

* write double bonds
      IF (dbflg.NE.0) CALL dwrite(tsmile)

* remove CdH hydrogens
* remove 'd'

60    IF(INDEX(tsmile,'d').NE.0) THEN
        ptr=INDEX(tsmile,'d')
        tsmile(ptr:lfo-1)=tsmile(ptr+1:lfo)
      ENDIF
      IF(INDEX(tsmile,'d').NE.0) GOTO 60

* ------------------------------------------------------
* RETURN THE STANDARDIZED SMILES FORMULA 
* ------------------------------------------------------

      smile = tsmile
      IF(INDEX(smile," ").EQ.1)THEN
        smile(1:lfo-1)=smile(2:lfo)
        smile(lfo:lfo)=" "
      ENDIF
       
!      PRINT*,"C_H_O_N_",ncbn,nhyd,noxy,nnit

! skip the parentheses check
      RETURN

* check parenthesis:if open parentheses,then stop the run.
      nc = INDEX(smile,' ') - 1
      p = 0
      DO i=1,nc
        IF(smile(i:i).EQ.'(') p = p + 1
        IF(smile(i:i).EQ.')') p = p - 1
      ENDDO

      IF(p.NE.0) THEN
        WRITE(6,'(a)') '--error-- 1'
        WRITE(6,'(a)') 'from routine : smile'
        WRITE(6,'(a)') 'parentheses mismatch in output SMILES:'
        WRITE(6,'(a)') chem
        WRITE(6,'(a)') smile
!        STOP
      ENDIF
     
      END
************************************************************************
* MASTER MECHANISM - ROUTINE NAME :      dwrite                        *
*                                                                      *
*                                                                      *
* PURPOSE :  Add "=" to copies with double bonds                       *
*                                                                      *
* INPUT/OUTPUT :                                                       *
*  - cp           : chemical formula in which '=' is added             *
************************************************************************
      SUBROUTINE dwrite(cp)
      IMPLICIT NONE
      INCLUDE 'smilesvars.h'

* input/output:
      CHARACTER*(lfo) cp

* internal:
      CHARACTER*(lfo) tempkc
      INTEGER         nc,idb,l,i

* initialize
      tempkc = ' '
      l   = 0
      idb = 0
      nc  = INDEX(cp,' ') - 1

* write "=" and reset IDB if not CdCdCd:
      DO i=1,nc
        IF(cp(i:i+1).EQ.'Cd') THEN
          idb = idb + 1
          IF (idb.GT.1) THEN
            l = l + 1
            tempkc(l:l) = '='
            IF (cp(i+2:i+3).NE.'Cd') idb = 0
          ENDIF
        ENDIF

* normal write:
        l = l + 1
        tempkc(l:l) = cp(i:i)
      ENDDO

* exit if formula is longer than lfo:
      IF (l.GT.lfo) THEN
        WRITE(6,'(a)') '--error--'
        WRITE(6,'(a)') 'from MASTER MECHANISM ROUTINE : dwrite'
        WRITE(6,'(a)') 'chemical formula is too long:'
        WRITE(6,'(a)') cp
        STOP
      ENDIF

* write-out formula and return:
      cp = tempkc
      RETURN
      END
************************************************************************
* MASTER MECHANISM - ROUTINE NAME : grbond                             *
*                                                                      *
* PURPOSE: Contruction of the group matrix and the carbon-carbon       *
*          bond matrix of a molecule given as input.                   *
*                                                                      *
* INPUT:                                                               *
* - chem         : chemical formula                                    *
* - nc           : number of characters in chem                        *
*                                                                      *
* OUTPUT:                                                              *
* - group(i)     : groups at position (carbon) i                       *
* - bond(i,j)    : carbon-carbon bond matrix of CHEM                   *
* - dbflg        : flag for double-bond indication                     *
* - nring        : number of separate rings in CHEM                    *
*                                                                      *
* First groups are constructed. A group starts with a carbon "C", an   *
* aromatic carbon "c", or an ether bond "-O-",                         *
* and ends with either a next "C" or "c", or a                         *
* double-bond "=", or an ether bond "-O-", or  blank "NUL". In each    *
* group the trailing parentheses are deleted if it is an opening       *
* with no closing and vice-versa.                                      * 
* Next the carbon skeleton is built with carbon, oxygen (if ether)     *
* and remaining parentheses and double-bonds.                          *
* The bond matrix is then built based on the skeleton.                 *
* The program checks the valence for each carbon center.               *
*                                                                      *
************************************************************************
      SUBROUTINE grbond(chem,nc,group,bond,dbflg,nring)
      IMPLICIT NONE
      INCLUDE 'smilesvars.h'

* input:
      CHARACTER*(lfo) chem
      INTEGER         nc

* output:
      CHARACTER*(lgr) group(mca)
      INTEGER         bond(mca,mca), dbflg
      INTEGER         ring(mca,mca), nring

* internal:
      CHARACTER*(lfo) skelet
      INTEGER         ig,ng,ik,istop,nb,tp,p
      INTEGER         i,j,k,l,nok,ii,jj,kk,n,nn
      INTEGER         dbbrk ! flag for chem where SMILES contains 'C=1'
      LOGICAL         loeter

* ----------
* INITIALIZE
* ----------
      !print*,'*grbond*',chem
      ig = 0
      ik = 0
      dbflg = 0
      nring = 0
      dbbrk = 0
      istop = 0
      l = 0
      skelet = ' '
      loeter=.false.

      group(:) = ' '

      bond(:,:) = 0
      ring(:,:) = 0

      IF (chem(1:1).NE.'C' .AND. chem(1:1).NE.'c' .AND. 
     &    chem(1:2).NE.'-O')THEN
        IF (chem(1:4).EQ.'=Cd1') THEN
          dbbrk=1
        ELSE
          WRITE(6,*)'WARNING : input to grbond must begin with C or c'
          WRITE(6,*)'begin with C/c:',chem
          STOP
        ENDIF
      ENDIF

* -------------------------------------------------------------
* CONSTRUCT CARBON-CENTERED GROUPS, SKELETON AND DIAGONAL TERM
* -------------------------------------------------------------
      DO 10 i=1,nc

* identify group
* --------------

* find where the group starts
        nok=0
        IF (chem(i:i).EQ.'C'.OR.chem(i:i).EQ.'c') nok=1
        IF (chem(i:i+1).EQ.'Cl') nok=0
        IF (chem(i:i+1).EQ.'-O') nok=1
        IF (nok.EQ.0) GO TO 10  ! cycle

        ig = ig + 1

* find where the group ends
        DO j=i+1,i+lgr
          nok=0
          IF (chem(j:j).EQ.'C'.OR.chem(j:j).EQ.'c')    nok=1
          IF (chem(j:j+1).EQ.'Cl')   nok=0
          IF (chem(j:j)  .EQ.'=')    nok=1
          IF (chem(j:j+1).EQ.'-O')  nok=1
          IF (chem(j:j)  .EQ.' ')    nok=1
          IF (nok.EQ.1) THEN
            l = j - i 
            istop = j - 1
            GO TO 20  ! exit
          ENDIF
        ENDDO

* write out group               
20      group(ig) = chem(i:istop)

* delete trailing parentheses from group
100     CONTINUE
        IF (group(ig)(l:l).EQ.'(') THEN
           group(ig)(l:l) = ' '
           l = l - 1
           GOTO 100
        ELSE IF (group(ig)(l:l) .EQ. ')') THEN
           p = 0
           DO j=1,l
             if (group(ig)(j:j) .eq. '(') p = p + 1
             if (group(ig)(j:j) .eq. ')') p = p - 1
           ENDDO
           IF (p .LT. 0) THEN
              group(ig)(l:l) = ' '
              l = l - 1
              GOTO 100
           ENDIF
        ENDIF

* build skeleton
* --------------
        ik = ik + 1
* skeletal oxygen
        IF (group(ig)(1:2).eq.'-O') THEN
          skelet(ik:ik) = 'O'
          j=3
* ring-joining oxygen (Oxygen is never a multiple center)
          DO n=1,mri
            IF(group(ig)(3:3).EQ.digit(n))THEN
              nring = nring+1
              ik=ik+1
              skelet(ik:ik)=digit(n)
              j=4
            ENDIF
          ENDDO
          group(ig)(j:j) = '-'
          loeter=.true.
        ELSE
* carbon (either flavor)
          IF(group(ig)(1:1).EQ.'c'.OR.group(ig)(1:1).EQ.'C')THEN
            skelet(ik:ik) = group(ig)(1:1)
            jj=2
            IF(group(ig)(2:2).EQ.'d') jj=3
*  ring-joining carbon
            DO 110 j=jj,l
              DO n=1,mri
                IF(group(ig)(j:j).EQ.digit(n))THEN
                  ik=ik+1
                  skelet(ik:ik)=group(ig)(j:j)
                  nring = nring+1
                  GOTO 110 ! try next character (in case multiple center)
                ENDIF
              ENDDO
! If reach this point, no more ring-joiners on group  
              GOTO 111  
110           CONTINUE
111         CONTINUE
          ENDIF

        ENDIF
        
* put parenthesis if any
        DO j=i+l,istop
          ik = ik + 1
          skelet(ik:ik) = chem(j:j)
        ENDDO

        IF (chem(istop+1:istop+1) .EQ. '=') THEN
          ik = ik + 1
          skelet(ik:ik) = '='
        ENDIF

* get the number of bonds at each node
* => construct diagonal of bond matrix
* (aromaticity is regarded as valence 3 for bookkeeping)
* --------------------------------------------
        nb = 0
        p = 0

        DO 120 j=2,l
* number of functional group inside parenthesis
          IF (group(ig)(j:j).EQ.'(' .AND. p .EQ. 0) nb=nb+1
          IF (group(ig)(j:j).EQ.'(') p = p + 1
          IF (group(ig)(j:j).EQ.')') p = p - 1

* H-,O-atom and radical attached to a carbon:
          IF (p .NE. 0) GOTO 120 ! cycle
          IF (group(ig)(j:j).EQ.'H')  nb = nb + 1
          IF (group(ig)(j-1:j-1).EQ.'H'.AND.
     &        group(ig)(j:j).EQ.'3')  nb = nb + 2
          IF (group(ig)(j-1:j-1).EQ.'H'.AND.
     &        group(ig)(j:j).EQ.'2')  nb = nb + 1
          IF (group(ig)(j:j).EQ.'O')  nb = nb + 2
          IF (group(ig)(j:j).EQ.'.')  nb = nb + 1 
120     CONTINUE

* assign to the diagonal term of the bond matrix
        bond(ig,ig) = nb

10    CONTINUE              

* keep number of groups:
      ng = ig

! troubleshooting
!      WRITE(6,'(15X,A11,A50)') 'skeleton = ',SKELET(1:50)
!      print*,' diagonal' 
!      DO j=1,ng
!        write(6,'(a10,30(i2))')group(j),(bond(i,j),i=1,ng)
!      ENDDO
!      print*,' '

      IF(MOD(nring,2).NE.0)THEN
        print*,'ERROR in grbond: RING NOT CLOSED'
        STOP
      ENDIF
      nring=nring/2

* ----------------------------------------------------------
* FIND TERMS OF BOND MATRIX (C-C, c-c, c-C, AND C-O-C BONDS)
* ----------------------------------------------------------

      ig = 0
      DO 50 i=1,ik-1
         nok=0
         IF (skelet(i:i).EQ.'C'.OR.
     &       skelet(i:i).EQ.'c'.OR.
     &       skelet(i:i).EQ.'O') nok=1
         IF (nok.EQ.0) GO TO 50  ! cycle

         ig = ig + 1
         k  = ig
         p  = 0
         tp = 0
         nb = 1
         DO 55 j=i+1,ik
           IF (skelet(j:j).EQ.'(') THEN
              tp = 1
              p  = p + 1
           !print*,'(    ',ig,k,nb,bond(ig,k)
           ELSE IF (skelet(j:j).EQ.')') THEN
              tp = 0
              p  = p - 1
           !print*,')    ',ig,k,nb,bond(ig,k)
              IF (p.LT.0) GOTO 50  ! exit
           ELSE IF (skelet(j:j).EQ.'C'.OR.skelet(j:j).EQ.'c') THEN
              k = k + 1
              IF (p.EQ.0 .AND. tp.EQ.0) THEN
                 bond(ig,k) = nb
                 bond(k,ig) = nb
           !print*,'C n()',ig,k,nb,bond(ig,k)
                 GOTO 50  ! exit
              ELSE IF (p.EQ.1 .AND. tp.EQ.1) THEN
                 bond(ig,k) = nb
                 bond(k,ig) = nb
           !print*,'C y()',ig,k,nb,bond(ig,k)
                 nb = 1
                 tp = 0 
                 GOTO 55  ! cycle 
! new section in case of double bonds up branches (2 lines of code new)
              ELSE IF (p.NE.tp) THEN
                nb = 1
           !print*,'p<>tp',ig,k,nb,bond(ig,k)
              ENDIF                       
           ELSE IF (skelet(j:j).EQ.'O') THEN
              k = k + 1
              IF (p.EQ.0 .AND. tp.EQ.0) THEN
                 bond(ig,k) = nb
                 bond(k,ig) = nb
                 GOTO 50  ! exit
              ELSE IF (p.EQ.1 .AND. tp.EQ.1) THEN
                 bond(ig,k) = nb
                 bond(k,ig) = nb
                 nb = 1
                 tp = 0 
                 GOTO 55  ! cycle 
              ENDIF                       
           ELSE IF (skelet(j-1:j).EQ.'(='.AND. p.EQ.1) THEN
             nb    = 2
             dbflg = 1
           ELSE IF (skelet(j:j).EQ.'=' .AND. p.EQ.0) THEN
             nb    = 2
             dbflg = 1
           ENDIF
55       CONTINUE
50    CONTINUE

      DO i=1,ng
        nb = 0
        DO j=1,mca
           nb = nb + bond(i,j)
        ENDDO

      ENDDO

!troubleshooting
      !print*,' basic skeleton' 
      !DO j=1,ng
      !  write(6,'(a10,30(i2))')group(j),(bond(i,j),i=1,ng)
      !ENDDO
      !print*,' '

! ----------------------------------------------------------
! FIND BOND MATRIX TERMS FOR RING-JOINING CARBONS
! ----------------------------------------------------------
! i  = group index of current node
! ii = group index of subsequent node 
! j  = skeleton index of current node
! jj = skeleton index of subsequent node
! k  = skeleton index of current ring-join character
! kk = skeleton index of subsequent ring-join character
! ----------------------------------------------------------
      IF(nring.GT.0)THEN
        i=0
        ii=0
        j=0
        jj=0
        k=0
        kk=0

        DO j=1,ik-1  ! loop skeleton
          IF (skelet(j:j).EQ.'C'.OR.
     &        skelet(j:j).EQ.'c'.OR.
     &        skelet(j:j).EQ.'O')THEN  ! node
            k=j
            i=i+1
            nb = 0
60          CONTINUE
            k=k+1
            DO n=1,nring  ! loop digits
              IF (skelet(k:k).EQ.digit(n)) THEN  ! ring start
                IF (ring(n,1).EQ.1) THEN ! ring already dealt with
                  GO TO 60  ! test for multi-ring center 
                ENDIF
                ii = i  
                nb = 1
                DO jj=k+1,ik  ! loop skeleton forwards
                   IF (skelet(jj:jj).EQ.'C'.OR.
     &                skelet(jj:jj).EQ.'c'.OR.
     &                skelet(jj:jj).EQ.'O')THEN  ! node
                    ii=ii+1 
                    kk = jj+1
                    IF(skelet(kk:kk).EQ.digit(n))THEN ! ring joined
                      !print*,"  ring ",digit(n)," joins at nodes", i,ii
                      ring(n,1)=1
                      bond(i,ii) = nb
                      bond(ii,i) = nb
                      IF(digit(n).EQ.'1'.AND.dbbrk.EQ.1)THEN
                        bond(i,ii) = bond(i,ii)+1
                        bond(ii,i) = bond(ii,i)+1
                      ENDIF
                      GO TO 60  ! test for multi-ring center on 1st node
                    ELSE ! test for multi-ring center on 2nd node
                      DO nn=1,nring
                        IF (skelet(kk:kk).EQ.digit(nn)) THEN  ! ring char
                          kk=jj+2
                          IF(skelet(kk:kk).EQ.digit(n))THEN ! ring joined
                      !print*,"  ring ",digit(n)," joins at nodes", i,ii
                            ring(n,1)=1
                            bond(i,ii) = nb
                            bond(ii,i) = nb
                            GO TO 60  ! test for multi-ring center on 1st node
                          ENDIF  ! ring joined
                        ENDIF  ! ring char
                      ENDDO  ! loop digits for ring char to ignore
                    ENDIF  ! ring joined
                  ENDIF  ! node
                ENDDO  ! loop skeleton forwards
              ENDIF  ! ring start
            ENDDO  ! loop digits
          ENDIF  ! node
        ENDDO  ! loop skeleton
      ENDIF  ! ring(s) present

! troubleshooting
      !print*,' after ring-joining' 
      !DO j=1,ng
      !  write(6,'(a10,30(i2))')group(j),(bond(i,j),i=1,ng)
      !ENDDO
      !print*,' '

* -----------
* FINAL CHECK
* -----------

* verify valence = 4 on each carbon, or '3' if aromatic carbon:
      DO i=1,ng
        nb = 0
        DO j=1,mca
           nb = nb + bond(i,j)
        ENDDO

        IF (group(i)(1:1).EQ.'C'.AND.nb.NE.4 .OR.
     &      group(i)(1:1).EQ.'c'.AND.nb.NE.3) THEN
          WRITE(6,'(a)') '--error-- '
          WRITE(6,'(a)') 'from SGMM routine : grbond'
          WRITE(6,'(a)') 'incorrect valence for species:'
          WRITE(6,'(a)') chem
          WRITE(6,'(15X,A11,A50)') 'skeleton = ',SKELET(1:50)
          WRITE(6,'(10X,A16)') 'bond - matrix = '
          DO k=1,ng
            WRITE(6,'(30(i2))') (bond(k,l),l=1,ng)
          ENDDO
          STOP
        ENDIF
        bond(i,i) = 0
      ENDDO
      !print*,'after zeroing main diagonal'
      !DO j=1,ng
      !  write(6,'(a10,#0(i2))')group(j),(bond(i,j),i=1,ng)
      !ENDDO
      !print*,' '

* check that no 'Cd' group exists without a Cd=Cd structure
      dbflg=0
      DO i=1,ng
        IF(INDEX(group(i),'Cd').NE.0)THEN
          DO j=1,mca
            IF(bond(i,j).EQ.2)dbflg=1
          ENDDO
          IF(dbflg.NE.1)THEN
            WRITE(6,'(a)') '--error-- '
            WRITE(6,'(a)') 'from SGMM routine : grbond'
            WRITE(6,'(a)') 'Cd but no >C=C< structure in species:'
            WRITE(6,'(a)') chem
            WRITE(6,'(15X,A11,A50)') 'skeleton = ',SKELET(1:50)
            WRITE(6,'(10X,A16)') 'bond - matrix = '
            DO k=1,ng
              WRITE(6,'(30(i2))') (bond(k,l),l=1,ng)
            ENDDO
            STOP
          ENDIF              
        ENDIF              
      ENDDO

* check that the -O- function is not a terminal position
      IF (loeter) THEN
        DO i=1,ng
          IF (group(i)(1:2).EQ.'-O') THEN
            nok=0
            DO j=1,ng
              nok=nok+bond(i,j)
            ENDDO
            IF (nok.NE.2) THEN
              WRITE(6,'(a)') '--error-- '
              WRITE(6,'(a)') 'from SGMM routine : grbond'
              WRITE(6,'(a)') 'wrong number of bond at an -O- site'
              WRITE(6,'(a)') 'valence should be 2 at that site.'
              WRITE(6,'(a)') 'chemical is :'
              WRITE(6,'(a)') chem
              WRITE(6,'(15X,A11,A50)') 'skeleton = ',SKELET(1:50)
              WRITE(6,'(10X,A16)') 'bond - matrix = '
              DO k=1,mca
                WRITE(6,'(30(1X,I2))') (bond(k,l),l=1,mca)
              ENDDO
              STOP
            ENDIF
          ENDIF
        ENDDO   

* C-O-C bonds turned to 3 (instead of 1) to be recognized later. 
        DO i=1,ng-1
          IF (group(i)(1:2).EQ.'-O') THEN
            DO j=1,ng
              IF (bond(i,j).EQ.1) THEN
                bond(i,j)=3
                bond(j,i)=3
              ENDIF
            ENDDO
          ENDIF
        ENDDO  
      ENDIF 

! troubleshooting
      !print*,'from grbond'
      !print*,'ng=',ng
      !DO j=1,ng
      !  write(6,'(a10,14(i2))')group(j),(bond(i,j),i=1,ng)
      !ENDDO
      !print*,' '

      RETURN
      END
************************************************************************
* MASTER MECHANISM - ROUTINE NAME : lntree                             *
*                                                                      *
*                                                                      *
* PURPOSE : Set-up the tree of the C-C and C-O-C bond starting at      *
*           the group in the input (top) and evaluate the longest      *
*           path and its length.                                       *
*                                                                      *
* INPUT:                                                               *
* - bond(i,j)    : carbon-carbon bond matrix                           *
* - dbflg        : flag for double-bond indication                     *
* - top          : 1st group (C) => starting point                     *
* - sec          : 2nd group (C) => carbon just after top in the       *
*                                   skeleton                           *
* - nca          : number of group                                     *
*                                                                      *
* INPUT/OUTPUT:                                                        *
* - clngth(i,j)  : length of jth path found starting at the ith        *
*                  group (C) position (here "i" is "top")              *
* - path(i,j,k)  : trees (path) of the jth path found starting at the  *
*                  ith group (C) position (here "i" is "top")          *
*                                                                      *
*   The purpose of this routine is to evaluate the longest chain of    *
*   the molecule. With the information in the bond-matrix, it is       *
*   possible to evaluate the top-down trees of the molecule:           *
*   e.g.                  CO(CH3)CH(CH(OH)(CH3))CHO                    *
*                         1   2   3   4     5    6                     *
*                   ___                                                *
*                  | 1 |  parent of 2,3                                *
*                   ---                                                *
*                 /     \                                              *
*             ___        ___                                           *
*  child of 1| 2 |      | 3 | child of 1, parent of 4,6                *
*             ---        ---                                           *
*                      /     \                                         *
*                   ___       ___                                      *
*   child of 3 &   | 4 |     | 6 |  child of 3                         *
*   parent of 5     ---       ---                                      *
*                 /                                                    *
*             ___                                                      *
*            | 5 |  child of 4                                         *
*             ---                                                      *
*                                                                      *
*   A child on the left hand is called "LEFT", on the right hand       *
*   "RIGHT" and in the middle "CENTER".                                *
*   In CLNGTH the length of each longest tree is stored.               *
*   Nevertheless the longest chain is the chain with most of the       *
*   double bonds in it.                                                *
*   First all relationships (parent - children) are evaluated. Then    *
*   the top-down paths are defined so that first a LEFT, if not        *
*   available, a RIGHT and then a CENTER is taken. As often as at      *
*   least one child exists the specified path is followed further on.  *
*   In the last section the longest paths and the paths with the       *
*   most double-bonds in it still remain and are given to the cal-     *
*   ling routine.                                                      *
*  NOTE: double-bond path is deactivated: use routine lntreeCd instead *
************************************************************************
      SUBROUTINE lntree(bond,top,sec,nca,clngth,path)
      IMPLICIT NONE
      INCLUDE 'smilesvars.h'

* input/output:
      INTEGER  clngth(mca,mca)
      INTEGER  path(mca,mca,mca)
      INTEGER  top,sec
      INTEGER  nca
      INTEGER  bond(mca,mca)

* internal:
      INTEGER  left(mca),right(mca),center(mca),parent(mca)
      INTEGER  flag(mca)
      INTEGER  ptr,knt,nknt,nct1,nct2
      INTEGER  maxlng,iend,i,j,k
      LOGICAL  tbond(mca,mca)

* -----------
* initialize
* -----------

      ptr = 0
      DO i=1,mca
        flag(i) = 0
        left(i) = 0
        right(i) = 0
        center(i) = 0
        parent(i) = 0
      ENDDO

* make a logical bond matrix (tbond only used to find parent and child)
      DO i=1,mca
        DO j=1,mca
          IF (bond(i,j).eq.0) THEN
            tbond(i,j) = .false.
          ELSE
            tbond(i,j) = .true.
          ENDIF
        ENDDO
      ENDDO

      left(top)      = sec
      parent(sec)    = top
      tbond(top,sec) = .false.
      tbond(sec,top) = .false.

* ---------------------------------------
* get the relationships (parent/children)
* ---------------------------------------

      DO k=1,nca
        nknt = 0
        DO i=1,nca
          knt = 0
          DO j=1,nca
            IF (tbond(i,j)) THEN
              knt = knt + 1
              ptr = j
            ENDIF
          ENDDO
          nknt = nknt + knt

* look for ith carbon with only one node, where parents do not
* exist: (ith carbon = child,jth carbon = parent)
          IF (knt.EQ.1) THEN
            IF ( (parent(i).EQ.0).and.(i.NE.top) ) THEN
              parent(i) = ptr
              tbond(i,ptr) = .false.
              tbond(ptr,i) = .false.
              IF(left(ptr).EQ.0) THEN
                left(ptr) = i
              ELSE IF(right(ptr).EQ.0) THEN
                right(ptr) = i
              ELSE IF(center(ptr).EQ.0) THEN
                center(ptr) = i
              ELSE

* if all children taken, error in bond-matrix:
                DO j=1,mca
                  WRITE(6,*) (bond(j,iend),iend=1,mca)
                ENDDO
                WRITE(6,'(a)') '--error--'
                WRITE(6,'(a)') 'from MASTER MECHANISM ROUTINE: lntree'
                WRITE(6,'(a)') 'no path possible, error in bonding'
                STOP
              ENDIF
            ENDIF
          ENDIF
        ENDDO
* do loop until bond-matrix is (0).
        IF (nknt.EQ.0) GOTO 35
      ENDDO

35    CONTINUE

* ---------------------------------------------
* define all top-down pathes starting at "top"
* ---------------------------------------------

      nct1 = nca - 1
      nct2 = nca + 4
      DO 40 i=1,nct1
        ptr = top
        path(top,i,1) = top
        DO j=2,nct2
          IF(flag(ptr).EQ.0) THEN
             IF(left(ptr).NE.0) THEN
                ptr = left(ptr)
                path(top,i,j) = ptr
             ELSE
                flag(ptr) = 1
             ENDIF
          ELSE IF(flag(ptr).EQ.1) THEN
             IF(right(ptr).NE.0) THEN
                ptr = right(ptr)
                path(top,i,j) = ptr
             ELSE 
                flag(ptr) = 2
             ENDIF
          ELSE IF(flag(ptr).EQ.2) THEN
             IF(center(ptr).NE.0) THEN
                ptr = center(ptr)
                path(top,i,j) = ptr
             ELSE
                flag(ptr) = 3
                flag(parent(ptr)) = flag(parent(ptr)) + 1
                GOTO 40
             ENDIF
          ELSE IF (flag(ptr).EQ.3) THEN
             flag(parent(ptr)) = flag(parent(ptr)) + 1
             GOTO 40
          ENDIF
        ENDDO
40    CONTINUE

* ---------------------
* get the longest path 
* ---------------------

        maxlng = 0
        DO i=1,nca
          DO j=1,nca
            IF (path(top,i,j).NE.0) clngth(top,i)=clngth(top,i) + 1
          ENDDO

          IF (clngth(top,i).GT.maxlng) THEN
             maxlng = clngth(top,i)
             iend=i-1
             DO j=1,iend
               clngth(top,j) = 0
             ENDDO

          ELSE IF (clngth(top,i).LT.maxlng) THEN
             clngth(top,i) = 0
          ENDIF
        ENDDO

* end of lntree
      RETURN
      END
************************************************************************
* MASTER MECHANISM - ROUTINE NAME : mkcopy                             *
*                                                                      *
*                                                                      *
* PURPOSE: make a copy of a molecule according to the longest tree.    *
*          On each call of mkcopy, the group ig is written in "copy".  *
*          At the same time and according to the bond-matrix, all      *
*          groups having a bond to ig (except the bond of              *
*          longest tree), are attached to it (using parentheses).      *
*                                                                      *
* Ramifications are written according some priority rules. A max of 2  *
* ramifications is expected and the formula must have the form :       *
*        longest_tree-Cig(branch1)(branch2)-longest_tree               *
* Since the branches may also contain ramification and functionalities,*
* each branch must be written in a unique way. This is done by         *
* subroutine getbrch. If the carbon ig 'carry' 2 branches, then        *
* branch1 and branch2 must be evaluated to know which branch has the   *
* highest priority and must be written first.                          *
*                                                                      *
* The structure of the subroutine is :                                 *
*  1 - write the group ig and check if branching exist at              *
*      the ig position                                                 *
*  2 - if no branching => return                                       *
*  3 - if only 1 branching => get and write the branch                 *
*  4 - if 2 branching => get each branches, evaluate the priority of   *
*       each branch and write them.                                    *
*  It is expected that in most cases, the branch will only be 1C long. *
*  This case is tested first, since there is only 1 way to write the   *
*  the branch.                                                         *
*                                                                      *
*                                                                      *
* INPUT:                                                               *
* - lobond(i,j)  : logical carbon-carbon bond matrix                   *
* - group(i)     : group at position (carbon) i                        *
* - rank(i)      : rank (priority) of group i (low = more important)   *
* - ig           : is next group to be written to the copy             *
* - pg and ng    : previous and next group of ig in the longest tree   *
* - nca          : total number of group in the molecule               *
*                                                                      *
* INPUT/OUTPUT:                                                        *
* - ptr1         : pointer in "copy", where to put next group          *
* - copy         : chemical formula under that is curently written     *
*                                                                      *
************************************************************************
      SUBROUTINE mkcopy(lobond,group,nca,rank,nring,ig,pg,ng,ptr1,copy)
      IMPLICIT NONE
      INCLUDE 'smilesvars.h'

* input
      LOGICAL         lobond(mca,mca)
      CHARACTER*(lgr) group(mca)
      INTEGER         rank(mca)
      INTEGER         nca,nring,ig,pg,ng

* input/output:
      CHARACTER*(lfo) copy
      INTEGER         ptr1

* internal:
      INTEGER ptr2,i
      INTEGER ialpha,ia1,ia2,ib1,ib2
      INTEGER ml1,ml2,nsor,maxpri
      CHARACTER*(lfo) tempbr(mca)
      CHARACTER*(lfo) brch1,brch2

      !print*,'*mkcopy*'
* -----------------------------------------------------------
* WRITE GROUP IG TO COPY !, omitting 'd' characters if present
* -----------------------------------------------------------
      !IF(INDEX(group(ig),'d').EQ.0)THEN
        ptr2 = ptr1 + INDEX(group(ig),' ') - 2
        copy(ptr1:ptr2) = group(ig)
      !ELSE
      !  ptr2 = ptr1 + INDEX(group(ig),' ') - 1
      !  copy(ptr1:ptr1) = group(ig)(1:1)
      !  copy(ptr1+1:ptr2) = group(ig)(3:INDEX(group(ig),' '))
      !ENDIF
      ptr1 = ptr2 + 1               

* -----------------------------------------------------------
* LOOK FOR ALL ATTACHED GROUPS TO GROUP IG, BUT NOT PG OR NG
* -----------------------------------------------------------

* search number attached group to IG in "alpha" position
      ialpha=0
      ia1=0
      ia2=0
      DO i=1,nca
        IF (lobond(ig,i)) THEN
          IF ( (i.NE.pg) .AND. (i.NE.ng) ) THEN
            ialpha=ialpha+1
            IF (ialpha.eq.1) ia1=i
            IF (ialpha.eq.2) ia2=i
          ENDIF
        ENDIF
      ENDDO

* -----------------------------------------------------------
* IF NO ALPHA GROUP, THEN RETURN
* -----------------------------------------------------------

      IF (ialpha.eq.0) THEN
       !print*,'no alpha ',copy
       RETURN
      ENDIF

* -----------------------------------------------------------
* ONE ALPHA GROUP
* -----------------------------------------------------------
      IF (ialpha.eq.1) THEN

* check if at least a beta position is occupied
        ib1=0
        DO i=1,nca
          IF (lobond(ia1,i)) THEN
            IF (i.ne.ig) THEN
              ib1=i
              GOTO 15
            ENDIF
          ENDIF
        ENDDO
15      CONTINUE

* no beta position found : write group alpha and return
        IF (ib1.eq.0) THEN
          ptr2=INDEX(group(ia1),' ') - 1
          ptr2 = ptr1 + ptr2
          copy(ptr1:ptr1)   = '('
          copy(ptr1+1:ptr2) = group(ia1)
          ptr1 = ptr2 + 1
          copy(ptr1:ptr1)=')'
          ptr1=ptr1+1
       !print*,'no beta ',copy
          RETURN

* at least a beta position found
        ELSE

* search the longest tree starting from ia1 with the highest priority
* write out
          CALL getbrch(lobond,group,nca,rank,nring,ig,ia1,ib1,brch1,ml1)
          ptr2=INDEX(brch1,' ') - 1
          ptr2 = ptr1 + ptr2
          copy(ptr1:ptr1)   = '('
          copy(ptr1+1:ptr2) = brch1
          ptr1 = ptr2 + 1
          copy(ptr1:ptr1)=')'
          ptr1=ptr1+1
       !print*,'yes beta ',copy
          RETURN
        ENDIF
      ENDIF

* -----------------------------------------------------------
* TWO ALPHA GROUP
* -----------------------------------------------------------

* 4 various cases need to be considered :
* case 1 : the 2 branches are C1 branches
* case 2 : branch 1 is a "long" branch, branch 2 is a C1 branch
* case 3 : branch 1 is a C1 branch, branch 2 is a "long" branch
* case 4 : the 2 branches are "long" branches

      IF (ialpha.eq.2) THEN

* check if at least a beta position is occupied
        ib1=0
        DO i=1,nca
          IF (lobond(ia1,i)) THEN
            IF (i.ne.ig) THEN
              ib1=i
              GOTO 115
            ENDIF
          ENDIF
        ENDDO
115     CONTINUE

        ib2=0
        DO i=1,nca
          IF (lobond(ia2,i)) THEN
            IF (i.ne.ig) THEN
              ib2=i
              GOTO 125
            ENDIF
          ENDIF
        ENDDO
125     CONTINUE

* get the branches starting at ia1 and ia2
        IF (ib1.ne.0) THEN
          CALL getbrch(lobond,group,nca,rank,nring,ig,ia1,ib1,brch1,ml1)
        ENDIF
        IF (ib2.ne.0) THEN
          CALL getbrch(lobond,group,nca,rank,nring,ig,ia2,ib2,brch2,ml2)
        ENDIF

* Case 1 :
* --------

* 2 methyl substitued => find the group with the highest priority
* and write the group having the lowest priority first, then return
        IF ((ib1.eq.0).AND.(ib2.eq.0)) THEN
          IF(rank(ia2).GE.rank(ia1))THEN
            nsor=ia1
          ELSE
            nsor=ia2
          ENDIF
          IF (nsor.EQ.2) THEN
            ptr2 = INDEX(group(ia1),' ') - 1
            ptr2 = ptr1 + ptr2
            copy(ptr1:ptr1)   = '('
            copy(ptr1+1:ptr2) = group(ia1)
            ptr1 = ptr2 + 1
            copy(ptr1:ptr1) = ')'
            ptr1 = ptr1 + 1

            ptr2 = INDEX(group(ia2),' ') - 1
            ptr2 = ptr1 + ptr2
            copy(ptr1:ptr1)   = '('
            copy(ptr1+1:ptr2) = group(ia2)
            ptr1 = ptr2 + 1
            copy(ptr1:ptr1) = ')'
            ptr1 = ptr1 + 1
          ELSE
            ptr2 = INDEX(group(ia2),' ') - 1
            ptr2 = ptr1 + ptr2
            copy(ptr1:ptr1)   = '('
            copy(ptr1+1:ptr2) = group(ia2)
            ptr1 = ptr2 + 1
            copy(ptr1:ptr1) = ')'
            ptr1 = ptr1 + 1

            ptr2 = INDEX(group(ia1),' ') - 1
            ptr2 = ptr1 + ptr2
            copy(ptr1:ptr1)   = '('
            copy(ptr1+1:ptr2) = group(ia1)
            ptr1 = ptr2 + 1
            copy(ptr1:ptr1) = ')'
            ptr1 = ptr1 + 1
          ENDIF
       !print*,'1 ',copy
          RETURN
        ENDIF


* Case 2 :
* --------

* ib2=0 : write first branch2 (methyl group) then branch 1 (long chain)
        IF ( (ib1.ne.0).and.(ib2.eq.0) ) THEN
          ptr2 = INDEX(group(ia2),' ') - 1
           ptr2 = ptr1 + ptr2
          copy(ptr1:ptr1)   = '('
          copy(ptr1+1:ptr2) = group(ia2)
          ptr1 = ptr2 + 1
          copy(ptr1:ptr1) = ')'
          ptr1 = ptr1 + 1

          ptr2=INDEX(brch1,' ') - 1
          ptr2 = ptr1 + ptr2
          copy(ptr1:ptr1)   = '('
          copy(ptr1+1:ptr2) = brch1
          ptr1 = ptr2 + 1
          copy(ptr1:ptr1)=')'
          ptr1=ptr1+1
       !print*,'2 ',copy
          RETURN
        ENDIF

* Case 3 :
* --------

* ib1=0 : write first branch1 (methyl group) then branch 2 (long chain)
        IF ( (ib1.eq.0).and.(ib2.ne.0) ) THEN
          ptr2 = INDEX(group(ia1),' ') - 1
          ptr2 = ptr1 + ptr2
          copy(ptr1:ptr1)   = '('
          copy(ptr1+1:ptr2) = group(ia1)
          ptr1 = ptr2 + 1
          copy(ptr1:ptr1) = ')'
          ptr1 = ptr1 + 1

          ptr2=INDEX(brch2,' ') - 1
          ptr2 = ptr1 + ptr2
          copy(ptr1:ptr1)   = '('
          copy(ptr1+1:ptr2) = brch2
          ptr1 = ptr2 + 1
          copy(ptr1:ptr1)=')'
          ptr1=ptr1+1
       !print*,'3 ',copy
          RETURN
        ENDIF

* Case 4 :
* --------

* ib1 and ib2 are both not equal to 0. Check the length of the chain (mli)
* and write the shortest chain first. If the chains are of the same
* length then call brpri to find priority.
        IF ( (ib1.ne.0).and.(ib2.ne.0) ) THEN
          IF (ml1.lt.ml2) THEN
            ptr2=INDEX(brch1,' ') - 1
            ptr2 = ptr1 + ptr2
            copy(ptr1:ptr1)   = '('
            copy(ptr1+1:ptr2) = brch1
            ptr1 = ptr2 + 1
            copy(ptr1:ptr1)=')'
            ptr1=ptr1+1

            ptr2=INDEX(brch2,' ') - 1
            ptr2 = ptr1 + ptr2
            copy(ptr1:ptr1)   = '('
            copy(ptr1+1:ptr2) = brch2
            ptr1 = ptr2 + 1
            copy(ptr1:ptr1)=')'
            ptr1=ptr1+1
       !print*,'4a ',copy
            RETURN

          ELSE IF (ml2.lt.ml1) THEN
            ptr2=INDEX(brch2,' ') - 1
            ptr2 = ptr1 + ptr2
            copy(ptr1:ptr1)   = '('
            copy(ptr1+1:ptr2) = brch2
            ptr1 = ptr2 + 1
            copy(ptr1:ptr1)=')'
            ptr1=ptr1+1

            ptr2=INDEX(brch1,' ') - 1
            ptr2 = ptr1 + ptr2
            copy(ptr1:ptr1)   = '('
            copy(ptr1+1:ptr2) = brch1
            ptr1 = ptr2 + 1
            copy(ptr1:ptr1)=')'
            ptr1=ptr1+1
       !print*,'4b ',copy
            RETURN

          ELSE
            IF (brch1.eq.brch2) THEN
              maxpri=1
            ELSE
              tempbr(1)=brch1
              tempbr(2)=brch2
              DO i=3,mca
                tempbr(i)=' '
              ENDDO
              CALL brpri(group,rank,tempbr,2,nring,maxpri)
            ENDIF
            IF (maxpri.eq.1) THEN
              ptr2=INDEX(brch1,' ') - 1
              ptr2 = ptr1 + ptr2
              copy(ptr1:ptr1)   = '('
              copy(ptr1+1:ptr2) = brch1
              ptr1 = ptr2 + 1
              copy(ptr1:ptr1)=')'
              ptr1=ptr1+1

              ptr2=INDEX(brch2,' ') - 1
              ptr2 = ptr1 + ptr2
              copy(ptr1:ptr1)   = '('
              copy(ptr1+1:ptr2) = brch2
              ptr1 = ptr2 + 1
              copy(ptr1:ptr1)=')'
              ptr1=ptr1+1
       !print*,'4c ',copy
              RETURN

            ELSE
              ptr2=INDEX(brch2,' ') - 1
              ptr2 = ptr1 + ptr2
              copy(ptr1:ptr1)   = '('
              copy(ptr1+1:ptr2) = brch2
              ptr1 = ptr2 + 1
              copy(ptr1:ptr1)=')'
              ptr1=ptr1+1

              ptr2=INDEX(brch1,' ') - 1
              ptr2 = ptr1 + ptr2
              copy(ptr1:ptr1)   = '('
              copy(ptr1+1:ptr2) = brch1
              ptr1 = ptr2 + 1
              copy(ptr1:ptr1)=')'
              ptr1=ptr1+1
       !print*,'4d ',copy
              RETURN
            ENDIF

          ENDIF
        ENDIF

      ENDIF

* end of mkcopy
      END
************************************************************************
* MASTER MECHANISM - ROUTINE NAME : ratings                            *
*                                                                      *
*  20 July 2005, J. Lee-Taylor, NCAR                                   *
*                                                                      *
* PURPOSE:  Rates groups in molecule based on node connections and     *
*           group types.  Small rating gives higher priority.          *
*           Default rating = '99999....' so is LOW priority.           *
*           Rating scale adapted from original prioty subroutine       * 
*                                                                      *
* INPUT:                                                               *
* - nca       : number of nodes in molecule                            *
* - nring        : number of rings in molecule                            *
* - group(i)  : character string for each node and attached functional *
*               groups                                                 *  
* - bond(i,j) : node-node bond matrix  ('node' = 'C','c','-O-')        *
*                                                                      *
* OUTPUT:                                                              *
* - rank(i)   : integer ranks for groups                               *
*                                                                      *
************************************************************************
      SUBROUTINE ratings(nca,group,bond,nring,rank)

      IMPLICIT NONE
      INCLUDE 'smilesvars.h'

!input: 
      INTEGER          nca,nring
      CHARACTER*(lgr)  group(mca)
      INTEGER          bond(mca,mca)

!internal:
      INTEGER          i,j,nrat,cntr
      INTEGER          ncx(nca)    ! n_connections on node
      INTEGER          rjg(mri,2)  ! ring-join group pairs
      PARAMETER (nrat=24)
      CHARACTER*(nrat) nines, maxm
      CHARACTER*(nrat) rating(mca)
      !CHARACTER*(1)    digit(10)
      !DATA digit /'1','2','3','4','5','6','7','8','9','0'/

! output:
      INTEGER          rank(mca)

!-----------------------------------------------------------------!
      !print*,'*ratings*'

! find # of adjacent nodes, convert to character
      DO i=1,nca
        ncx(i)=0
        DO j=1,nca
          IF(bond(i,j).GT.0) ncx(i)=ncx(i)+1
        ENDDO
        rating(i)(1:1) =  digit(ncx(i))
      ENDDO

! remove ring-joining characters from groups if rings exist
      IF(nring.GT.0) CALL rjgrm(nring,group,rjg)

      nines = '999999999999999999999999'
      DO i=1,nca
        rating(i)(2:nrat) = nines(1:nrat-1)
!rating(i)(1:1) reserved for # non-H bonds
        IF(INDEX(group(i),'.' ).NE.0)         rating(i)(2:2) = '1'
        IF(INDEX(group(i),'CdO' ).NE.0)       rating(i)(4:4) = '1'
        IF(INDEX(group(i),'Cd' ).NE.0)        rating(i)(5:5) = '3'
        IF(INDEX(group(i),'CdH' ).NE.0)        rating(i)(5:5) = '2'
        IF(INDEX(group(i),'CdH2' ).NE.0)        rating(i)(5:5) = '1'
        IF(INDEX(group(i),'-O-' ).NE.0)       rating(i)(6:6) = '1'
        IF(INDEX(group(i),'CHO' )  .NE.0)     rating(i)(7:7) = '1'
        IF(INDEX(group(i),'CO(OONO2)' ).NE.0) rating(i)(8:8) = '1'
        IF(INDEX(group(i),'CO(OOH)' ).NE.0)   rating(i)(9:9) = '1'
        IF(INDEX(group(i),'(OOH)' ).NE.0)     rating(i)(10:10) = '3'
        IF(INDEX(group(i),'(OOH)(OOH)' ).NE.0) rating(i)(10:10) = '2'
        IF(INDEX(group(i),'(OOH)(OOH)(OOH)').NE.0)rating(i)(10:10) = '1'
        IF(INDEX(group(i),'(ONO2)').NE.0)     rating(i)(11:11) = '3'
        IF(INDEX(group(i),'(ONO2)(ONO2)').NE.0)rating(i)(11:11) = '2'
        IF(INDEX(group(i),'(ONO2)(ONO2)(ONO2)').NE.0) 
     &                                          rating(i)(11:11) = '1'
        IF(INDEX(group(i),'CO(OH)' ).NE.0)    rating(i)(12:12) = '1'
        IF(INDEX(group(i),'CO' ).NE.0)        rating(i)(13:13) = '1'
        IF(INDEX(group(i),'F' ).NE.0)         rating(i)(14:14) = '1'
        IF(INDEX(group(i),'Br' ).NE.0)        rating(i)(15:15) = '1'
        IF(INDEX(group(i),'Cl' ).NE.0)        rating(i)(16:16) = '1'
        IF(INDEX(group(i),'S' ).NE.0)         rating(i)(17:17) = '1'
        IF(INDEX(group(i),'NH' ).NE.0)        rating(i)(18:18) = '1'
        IF(INDEX(group(i),'(NO2)' ).NE.0)     rating(i)(19:19) = '3'
        IF(INDEX(group(i),'(NO2)(NO2)' ).NE.0) rating(i)(19:19) = '2' 
        IF(INDEX(group(i),'(NO2)(NO2)(NO2)').NE.0)rating(i)(19:19) = '1'
        IF(INDEX(group(i),'NO' ).NE.0)        rating(i)(20:20) = '1'
        IF(INDEX(group(i),'(OH)' ).NE.0)      rating(i)(21:21) = '3'
        IF(INDEX(group(i),'(OH)(OH)' ).NE.0)   rating(i)(21:21) = '2'
        IF(INDEX(group(i),'(OH)(OH)(OH)' ).NE.0) rating(i)(21:21) = '1'
        IF(INDEX(group(i),'C' ).NE.0)         rating(i)(22:22) = '4'
        IF(INDEX(group(i),'CH' ).NE.0)         rating(i)(22:22) = '3'
        IF(INDEX(group(i),'CH2' ).NE.0)         rating(i)(22:22) = '2'
        IF(INDEX(group(i),'CH3' ).NE.0)          rating(i)(22:22) = '1'
        IF(INDEX(group(i),'c' ).NE.0)         rating(i)(23:23) = '2'
        IF(INDEX(group(i),'cH' ).NE.0)         rating(i)(23:23) = '1'
      ENDDO

      cntr = 0
      nines = '999999999999999999999999'
      DO i = 1, nca !-1
        maxm = nines
        DO j = 1, nca
          IF (rating(j).LT.maxm) maxm = rating(j)
        ENDDO
        IF (maxm.eq.nines) GOTO 20  ! EXIT
        cntr = cntr + 1
        DO j = 1, nca
          IF (rating(j).LE.maxm) THEN
             rating(j) = nines
             rank(j) = cntr
          ENDIF
        ENDDO
      ENDDO
20    CONTINUE

      DO i = 1, nca
        IF(rank(i).EQ.0)THEN
          print*,'rank incorrectly assigned'
          STOP
        ENDIF
      ENDDO

! add ring-joining characters if rings exist
      IF (nring.GT.0) CALL rjgadd(nring,group,rjg)

! use priduct of adjacent primes to find unique ranks
      CALL primes(nca,bond,rank)

      RETURN
      END
!--------------------------------------------------------------------------!
! MASTER MECHANISM - ROUTINE NAME : rjggadd                                 !
!                                                                          !
! PURPOSE: Add ring-joining numerical characters to group strings          !
!          at nodes given by rjg.                                           !
!                                                                          !
! INPUT:                                                                   !
! - nring       : number of rings in chem                                  !
! - rjg(j,2)     : group numbers of ring-joining pairs for each ring        !
!                                                                          !
! IN/OUT:                                                                  !
! - group(i)    : groups at position (node) i                              !
!                                                                          !
!--------------------------------------------------------------------------!
      SUBROUTINE rjgadd(nring,group,rjg)
      IMPLICIT NONE
      INCLUDE 'smilesvars.h'

! input:
      INTEGER    nring
      INTEGER    rjg(mri,2)
! in/out:
      CHARACTER*(lgr) group(mca)
! internal:
      INTEGER    n,i,ii,j,k

!--------------------------------------------------------------------------!
      !print*,'*rjgadd*'
! loop in reverse order => chars in numerical order if >1 exist at any node
      DO n=nring,1,-1
        DO ii=1,2
          i = rjg(n,ii)
          IF(group(i)(1:1).EQ.' ') GO TO 10
          IF(group(i)(1:2).EQ.'-O'.OR.group(i)(2:2).EQ.'d')THEN
            j=3
          ELSE
            j=2
          ENDIF
          DO k=lgr,j+1,-1
            group(i)(k:k)=group(i)(k-1:k-1)
          ENDDO
          group(i)(j:j) = digit(n)
        ENDDO
10      CONTINUE
      ENDDO

      END
!--------------------------------------------------------------------------!
! MASTER MECHANISM - ROUTINE NAME : rjgrm                                 !
!                                                                          !
! PURPOSE: Strip ring-joining numerical characters from group strings.     !
!          Subroutine rjgadd puts them back.                               !
!                                                                          !
! INPUT:                                                                   !
! - nring       : number of rings in chem                                  !
!                                                                          !
! IN/OUT:                                                                  !
! - group(i)    : groups at position (node) i                              !
!                                                                          !
! OUTPUT:                                                                  !
! - rjg(j,2)     : group numbers of ring-joining pairs for each ring        !
!                                                                          !
!--------------------------------------------------------------------------!
      SUBROUTINE rjgrm(nring,group,rjg)
      IMPLICIT NONE
      INCLUDE 'smilesvars.h'

! input:
      INTEGER    nring
! in/out:
      CHARACTER*(lgr) group(mca)
! output:
      INTEGER    rjg(mri,2)
! internal:
      INTEGER    n,i,j,k

!--------------------------------------------------------------------------!
      !print*,'*rjgrm*'

      DO n=1,nring
        rjg(n,1)=0
        rjg(n,2)=0
        DO i=1,mca
!          IF(group(i)(1:1).EQ.' ') GO TO 10  
! causes problems for rings that have expelled products from branches
          IF(group(i)(1:2).EQ.'-O'.OR.group(i)(2:2).EQ.'d')THEN
            j=3
          ELSE
            j=2
          ENDIF
          IF(group(i)(j:j).EQ.digit(n)) THEN
            DO k=j,lgr-1
              group(i)(k:k)=group(i)(k+1:k+1)
            ENDDO
            IF(rjg(n,1).EQ.0)THEN
              rjg(n,1) = i
            ELSE
              rjg(n,2) = i
            ENDIF
          ENDIF
        ENDDO
      ENDDO
      !DO i=1,10
      !  print*,group(i)
      !ENDDO

      END
!-----------------------------------------------------------------!
! PURPOSE: Find unique artificial breakpoints for cyclic molecule !
!          including multi-ring compounds.                        !
! CREATED: June/July 2005, Julia Lee-TAYLOR, NCAR                 !
!-----------------------------------------------------------------!
      SUBROUTINE uniqring(nring,nca,group,bond,rank,rjg)
      IMPLICIT NONE
      include 'smilesvars.h'

! input: 
      INTEGER    nca,nring
      CHARACTER*(lgr) group(mca)
! i/o:
      INTEGER    bond(mca,mca)
      INTEGER    rank(mca)  ! rank of nodes, in order supplied
! output: 
      INTEGER    rjg(mri,2) ! ring-join group pairs

! internal:
      INTEGER    ppb(mca,mca) ! product of primes of nodes in bond
      INTEGER    cprim(mca)   ! prime corresponding to node's rank
      INTEGER    tbond(mca,mca) ! working bond matrix.  Used twice:
                 ! zero 'boken' bonds; mirror for rearrangmenet
      INTEGER    i,j,ii,jj,k,maxp,n,rngflg
      INTEGER    top,sec      ! first and second nodes in new arrangement
      INTEGER    parent(mca),child(mca,3),nk(mca)
      INTEGER    iold(mca)    ! old index of group whose new index is 'k'
      INTEGER    trjg(nring,2)  ! temporary ring-join group pairs
      INTEGER    trank(mca)   ! temporary rank listing, for rearrangement
      CHARACTER*(lgr) tgroup(mca) ! temporary group listing, for rearr.
      INTEGER    prim(30)    ! prime numbers
      INTEGER    rgpath(mca)
      DATA prim/  2,  3,  5,  7, 11, 13, 17, 19, 23, 29, 
     &           31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 
     &           73, 79, 83, 89, 97,101,103,107,109,113/

!---------------------------
      !print*,'*uniqring*'

! initialise
      ii = 1
      DO n=1,nring
        rjg(n,1)=0
        rjg(n,2)=0
      ENDDO
      DO i=1,nca
        tgroup(i)=' '
        DO j=1,nca
          tbond(i,j)=0
        ENDDO
      ENDDO

! find product of connected primes for actual bonds, setup connections 
      DO i=1,nca
        cprim(i)=prim(rank(i))
      ENDDO

      DO i=1,nca
        DO j=1,nca
          ppb(i,j) = 0
          tbond(i,j)=bond(i,j)
          IF(i.NE.j.AND.bond(i,j).GT.0) THEN
            ppb(i,j)=cprim(i)*cprim(j)
          ENDIF
        ENDDO
      ENDDO

! troubleshooting
      !DO j=1,nca
      !  write(6,'(a12,14(i2))')group(j),(tbond(i,j),i=1,nca)
      !ENDDO
      !print*,'primes bond matrix'
      !DO j=1,nca
      !  write(6,'(10(i4))') (ppb(i,j),i=1,nca)
      !ENDDO
      !print*,' '

! remove ring-joining characters from groups
      CALL rjgrm(nring,group,rjg)
! loop rings, finding lowest-priority on-ring connection to 'break' 
      DO n=1,nring
100     maxp = 0
        DO i=1,nca
          DO j=i+1,nca
! disallow breaking @ double bonds
            IF(bond(i,j).NE.2)THEN
              IF(ppb(i,j).GT.maxp) THEN
                ii = i
                jj = j
                maxp=ppb(ii,jj) 
              ENDIF
            ENDIF
          ENDDO
        ENDDO
        CALL findring(ii,jj,nca,tbond,rngflg,rgpath)
        ppb(ii,jj) = 0
        IF(rngflg.EQ.1) THEN ! connection is on a ring
          !print*,'breaking ring at',ii,jj
          tbond(ii,jj) = 0
          tbond(jj,ii) = 0
          rjg(n,1) = ii
          rjg(n,2) = jj
* symmetry might be broken after first bond breaking.
* therefore, call ratings to update ranks.          
          !IF (nring.GT.1) print*,"nring = ",nring,n
          IF (nring.GT.1) CALL ratings(nca,group,tbond,nring,rank)
        ELSE
          !print*,ii,jj,' not on a ring'
          GOTO 100
        ENDIF
      ENDDO ! n_rings

! find first 2 nodes in tree (linearly-expressed molecule)
      top = ii
      sec = 0
      DO j = 2,nca
        DO i = 1,j-1
          IF(tbond(i,j).GT.0)THEN
            !print*,i,j,tbond(i,j)
            IF(i.EQ.ii) sec=j
            IF(j.EQ.ii) sec=i
            IF(sec.GT.0) GOTO 200
          ENDIF
        ENDDO
      ENDDO

! rearrange into a connected skeleton (finding longest path comes later)
200   CONTINUE
      CALL findtree(tbond,top,sec,nca,parent,child)

! troubleshooting
      !DO i=1,nca
      !  print*,i,(child(i,k),k=1,3),parent(i)
      !ENDDO
      !print*,' '

      DO i=1,nca
        nk(i)=0
      ENDDO
      j=1
      iold(j)=top
      !  print*,iold(j),'old',0,'top',j,'top'
      IF(parent(iold(j)).NE.0)THEN
        !print*,'ERROR in uniqring : top has parent!'
        STOP
      ENDIF

      i=j
210   CONTINUE
      k = nk(i)+1
      IF(k.LE.3) THEN
        IF(child(iold(i),k).GT.0) THEN
!       print*,'a ',i,nk(i),j,iold(j),iold(i),k
!     &        ,parent(iold(i)),child(iold(i),k)
          nk(i) = nk(i)+1
          j=j+1
          iold(j)=child(iold(i),k)
          i=j ! step forward
          GO TO 210 ! try for next child
        ENDIF
      ENDIF
! otherwise, no further children on node
!       print*,'b ',i,nk(i),j,iold(j),iold(i),k
!     &        ,parent(iold(i)),child(iold(i),k)
      IF(parent(iold(i)).EQ.0) GO TO 212 ! finished thoroughly checking molecule
      DO ii=1,nca
        IF(parent(iold(i)).EQ.iold(ii))THEN
          i = ii
          GO TO 210 ! backtrack to parent
        ENDIF
      ENDDO
212   CONTINUE
      !print*,'finished sorting new tree'

! reassign arrays based on new ordering
      DO ii=1,nca
        IF(iold(ii).NE.0)THEN
          trank(ii) = rank(iold(ii))
          tgroup(ii) = group(iold(ii))
        ENDIF
        DO jj=1,nca
          IF(iold(jj).NE.0)THEN
            tbond(ii,jj) = bond(iold(ii),iold(jj))
          ELSE
! retain zeroes for broken bonds (e.g. for fragmentation that retains ring)
            tbond(ii,jj)=0
          ENDIF
        ENDDO
        DO n=1,nring
          DO i=1,2
            IF(rjg(n,i).EQ.iold(ii)) THEN
              trjg(n,i)=ii
            ENDIF
          ENDDO
        ENDDO
      ENDDO

! b: replace working arrays
      DO ii=1,nca
        rank(ii) = trank(ii)
        group(ii) = tgroup(ii)
        DO jj=1,nca
          bond(ii,jj) = tbond(ii,jj)
        ENDDO
      ENDDO
      DO n=1,nring
        rjg(n,1)=MIN0(trjg(n,1),trjg(n,2))
        rjg(n,2)=MAX0(trjg(n,1),trjg(n,2))
      ENDDO
     
! add ring-joining characters 
      CALL rjgadd(nring,group,rjg)

! troubleshooting
      !print*,'new BM '
      !DO j=1,nca
      !  write(6,'(a12,14(i2))')group(j),(bond(i,j),i=1,nca)
      !ENDDO
      !DO j=1,nca
      !  write(6,*)group(j),rank(j)
      !ENDDO
      !print*,''

      !print*,'*end uniqring*'

      RETURN
      END
